use crate::mcp_client::MCPClient;
use crate::prompt::Prompt;
use std::error::Error;
use std::path::Path;

pub struct CommandHandler {
    client: MCPClient,
    pub prompt: Prompt,
}

impl CommandHandler {
    pub fn new(client: MCPClient, prompt: Prompt) -> Self {
        Self { client, prompt }
    }
    
    pub async fn handle(&self, input: &str) -> Result<(), Box<dyn Error>> {
        let parts: Vec<&str> = input.split_whitespace().collect();
        
        if parts.is_empty() {
            return Ok(());
        }
        
        // Check for special commands
        match parts[0] {
            "/help" | "/h" => self.show_help(),
            "/clear" => self.clear_screen(),
            "/model" => self.handle_model_command(&parts).await?,
            "/analyze" => self.handle_analyze_command(&parts).await?,
            "/create" => self.handle_create_command(&parts).await?,
            _ => {
                // Regular prompt - send to LLM
                self.handle_prompt(input).await?;
            }
        }
        
        Ok(())
    }
    
    fn show_help(&self) {
        self.prompt.info("Available commands:");
        println!("  /help, /h     - Show this help message");
        println!("  /clear        - Clear the screen");
        println!("  /model [name] - Switch LLM model");
        println!("  /analyze <file> - Analyze a file");
        println!("  /create <type> <name> - Create a new project/file");
        println!("  exit, quit    - Exit the program");
        println!("\nJust type your request to interact with AI");
        println!("\nExamples:");
        println!("  create a web server in rust");
        println!("  explain this error: [paste error]");
        println!("  /analyze src/main.rs");
        println!("  /create rust my-project");
    }
    
    fn clear_screen(&self) {
        print!("\x1B[2J\x1B[1;1H");
    }
    
    async fn handle_model_command(&self, parts: &[&str]) -> Result<(), Box<dyn Error>> {
        if parts.len() < 2 {
            self.prompt.info("Current model: qwen2.5-coder:7b"); // TODO: Get from config
            self.prompt.info("Available models: qwen2.5-coder:7b, codellama:7b, deepseek-coder:6.7b");
        } else {
            // TODO: Implement model switching
            self.prompt.success(&format!("Switched to model: {}", parts[1]));
        }
        Ok(())
    }
    
    async fn handle_analyze_command(&self, parts: &[&str]) -> Result<(), Box<dyn Error>> {
        if parts.len() < 2 {
            self.prompt.error("Usage: /analyze <file>");
            return Ok(());
        }
        
        let file_path = parts[1];
        if !Path::new(file_path).exists() {
            self.prompt.error(&format!("File not found: {}", file_path));
            return Ok(());
        }
        
        self.prompt.info(&format!("Analyzing file: {}", file_path));
        
        match self.client.analyze_file(file_path).await {
            Ok(analysis) => {
                self.prompt.success("Analysis complete:");
                println!("{}", analysis);
            }
            Err(e) => {
                self.prompt.error(&format!("Analysis failed: {}", e));
            }
        }
        
        Ok(())
    }
    
    async fn handle_create_command(&self, parts: &[&str]) -> Result<(), Box<dyn Error>> {
        if parts.len() < 3 {
            self.prompt.error("Usage: /create <type> <name>");
            self.prompt.info("Types: rust, python, node, react");
            return Ok(());
        }
        
        let project_type = parts[1];
        let project_name = parts[2];
        
        let prompt_text = match project_type {
            "rust" => format!("Create a new Rust project named '{}' with a basic structure including Cargo.toml, src/main.rs, and README.md", project_name),
            "python" => format!("Create a new Python project named '{}' with setup.py, requirements.txt, src/__init__.py, and README.md", project_name),
            "node" => format!("Create a new Node.js project named '{}' with package.json, index.js, and README.md", project_name),
            "react" => format!("Create a new React app structure for '{}' with basic components", project_name),
            _ => {
                self.prompt.error(&format!("Unknown project type: {}", project_type));
                return Ok(());
            }
        };
        
        self.prompt.info(&format!("Creating {} project: {}", project_type, project_name));
        self.handle_prompt(&prompt_text).await?;
        
        Ok(())
    }
    
    async fn handle_prompt(&self, prompt: &str) -> Result<(), Box<dyn Error>> {
        self.prompt.info("Processing your request...");
        
        // Detect the context and language
        let language = self.detect_language(prompt);
        
        match self.client.generate_code(prompt, &language).await {
            Ok(code) => {
                if !code.is_empty() {
                    self.prompt.success("Generated code:");
                    self.prompt.code_block(&code, &language);
                    
                    // Ask if user wants to save the code
                    self.prompt.info("Would you like to save this code? (y/n)");
                    if let Ok(response) = self.prompt.read_input() {
                        if response.to_lowercase() == "y" {
                            // TODO: Implement file saving logic
                            self.prompt.info("Enter filename:");
                            if let Ok(filename) = self.prompt.read_input() {
                                match std::fs::write(&filename, &code) {
                                    Ok(_) => self.prompt.success(&format!("Code saved to: {}", filename)),
                                    Err(e) => self.prompt.error(&format!("Failed to save file: {}", e)),
                                }
                            }
                        }
                    }
                } else {
                    self.prompt.info("No code was generated. The AI might have provided an explanation instead.");
                }
            }
            Err(e) => {
                self.prompt.error(&format!("Failed to generate code: {}", e));
            }
        }
        
        Ok(())
    }
    
    fn detect_language(&self, prompt: &str) -> String {
        let prompt_lower = prompt.to_lowercase();
        
        if prompt_lower.contains("rust") || prompt_lower.contains("cargo") {
            "rust".to_string()
        } else if prompt_lower.contains("python") || prompt_lower.contains("pip") {
            "python".to_string()
        } else if prompt_lower.contains("javascript") || prompt_lower.contains("node") || prompt_lower.contains("npm") {
            "javascript".to_string()
        } else if prompt_lower.contains("typescript") {
            "typescript".to_string()
        } else if prompt_lower.contains("go") || prompt_lower.contains("golang") {
            "go".to_string()
        } else if prompt_lower.contains("java") && !prompt_lower.contains("javascript") {
            "java".to_string()
        } else if prompt_lower.contains("c++") || prompt_lower.contains("cpp") {
            "cpp".to_string()
        } else if prompt_lower.contains("c#") || prompt_lower.contains("csharp") {
            "csharp".to_string()
        } else {
            "text".to_string()
        }
    }
}