use serde::{Deserialize, Serialize};
use std::path::PathBuf;
use dirs::home_dir;

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Config {
    pub server: ServerConfig,
    pub llm: LLMConfig,
    pub logging: LoggingConfig,
    pub cli: CLIConfig,
    pub security: SecurityConfig,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct ServerConfig {
    pub host: String,
    pub port: u16,
    pub protocol: String,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct LLMConfig {
    pub ollama_host: String,
    pub default_model: String,
    pub timeout_seconds: u64,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct LoggingConfig {
    pub level: String,
    pub file: String,
    pub max_size_mb: u64,
    pub max_backups: u32,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct CLIConfig {
    pub history_file: String,
    pub max_history: usize,
    pub prompt: String,
    pub color_output: bool,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct SecurityConfig {
    pub allowed_commands: Vec<String>,
    pub sandbox_mode: bool,
    pub max_file_size_mb: u64,
}

impl Config {
    pub fn load() -> Result<Self, Box<dyn std::error::Error>> {
        let config_path = Self::config_path();
        
        if config_path.exists() {
            let content = std::fs::read_to_string(&config_path)?;
            let config: Config = toml::from_str(&content)?;
            Ok(config)
        } else {
            Ok(Self::default())
        }
    }
    
    pub fn config_path() -> PathBuf {
        home_dir()
            .unwrap_or_else(|| PathBuf::from("."))
            .join(".config")
            .join("syui")
            .join("ai")
            .join("shell")
            .join("config.toml")
    }
    
    pub fn expand_path(path: &str) -> PathBuf {
        if path.starts_with("~") {
            home_dir()
                .unwrap_or_else(|| PathBuf::from("."))
                .join(path.trim_start_matches("~/"))
        } else {
            PathBuf::from(path)
        }
    }
    
    pub fn server_url(&self) -> String {
        format!("{}://{}:{}", self.server.protocol, self.server.host, self.server.port)
    }
}

impl Default for Config {
    fn default() -> Self {
        Self {
            server: ServerConfig {
                host: "127.0.0.1".to_string(),
                port: 8765,
                protocol: "http".to_string(),
            },
            llm: LLMConfig {
                ollama_host: "http://localhost:11434".to_string(),
                default_model: "qwen2.5-coder:7b".to_string(),
                timeout_seconds: 300,
            },
            logging: LoggingConfig {
                level: "info".to_string(),
                file: "~/.config/syui/ai/shell/aishell.log".to_string(),
                max_size_mb: 10,
                max_backups: 3,
            },
            cli: CLIConfig {
                history_file: "~/.config/syui/ai/shell/history".to_string(),
                max_history: 1000,
                prompt: "ai> ".to_string(),
                color_output: true,
            },
            security: SecurityConfig {
                allowed_commands: vec![
                    "ls".to_string(),
                    "cat".to_string(),
                    "grep".to_string(),
                    "find".to_string(),
                    "git".to_string(),
                    "cargo".to_string(),
                    "npm".to_string(),
                    "python".to_string(),
                ],
                sandbox_mode: false,
                max_file_size_mb: 50,
            },
        }
    }
}