use clap::{Parser, Subcommand};
use colored::*;
use tokio;

use aishell::{Config, MCPClient, Prompt};
use aishell::commands::CommandHandler;

#[derive(Parser)]
#[command(name = "aishell")]
#[command(about = "AI-powered shell for code generation and automation")]
#[command(version = "0.1.0")]
struct Cli {
    #[command(subcommand)]
    command: Option<Commands>,
    
    /// Configuration file path
    #[arg(short, long)]
    config: Option<String>,
    
    /// Run in TUI mode (future feature)
    #[arg(long)]
    tui: bool,
}

#[derive(Subcommand)]
enum Commands {
    /// Start interactive session
    Chat,
    /// Execute a single command
    Exec { 
        /// The prompt to execute
        prompt: String 
    },
    /// Analyze a file
    Analyze { 
        /// Path to the file to analyze
        file: String 
    },
    /// Check server health
    Health,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let cli = Cli::parse();
    
    if cli.tui {
        eprintln!("TUI mode is not yet implemented");
        return Ok(());
    }
    
    // Load configuration
    let config = Config::load()?;
    
    // Initialize MCP client
    let client = MCPClient::new(&config.server_url());
    
    // Check server health
    if !check_server_health(&client).await {
        eprintln!("{}", "Error: MCP server is not running".red());
        eprintln!("Please start the server with: make run-server");
        return Ok(());
    }
    
    match cli.command {
        None | Some(Commands::Chat) => interactive_mode(client, config).await,
        Some(Commands::Exec { prompt }) => execute_once(client, config, &prompt).await,
        Some(Commands::Analyze { file }) => analyze_file(client, config, &file).await,
        Some(Commands::Health) => {
            println!("{}", "MCP server is healthy".green());
            Ok(())
        }
    }
}

async fn check_server_health(_client: &MCPClient) -> bool {
    // Use a simple HTTP GET request for health check
    match reqwest::get("http://localhost:8765/health").await {
        Ok(response) => {
            if response.status().is_success() {
                match response.json::<serde_json::Value>().await {
                    Ok(json) => json["status"] == "ok",
                    Err(_) => false,
                }
            } else {
                false
            }
        }
        Err(_) => false,
    }
}

async fn interactive_mode(client: MCPClient, config: Config) -> Result<(), Box<dyn std::error::Error>> {
    println!("{}", "AI Shell v0.1.0".bright_blue().bold());
    println!("Type '/help' for commands, 'exit' to quit\n");
    
    let prompt = Prompt::new(config.cli.prompt.clone(), config.cli.color_output);
    let handler = CommandHandler::new(client, prompt);
    
    loop {
        handler.prompt.display()?;
        
        let input = handler.prompt.read_input()?;
        
        match input.as_str() {
            "exit" | "quit" => {
                handler.prompt.info("Goodbye!");
                break;
            }
            "" => continue,
            _ => {
                if let Err(e) = handler.handle(&input).await {
                    handler.prompt.error(&format!("Error: {}", e));
                }
            }
        }
    }
    
    Ok(())
}

async fn execute_once(client: MCPClient, config: Config, prompt_text: &str) -> Result<(), Box<dyn std::error::Error>> {
    let prompt = Prompt::new(config.cli.prompt, config.cli.color_output);
    prompt.info(&format!("Executing: {}", prompt_text));
    
    match client.generate_code(prompt_text, "auto").await {
        Ok(code) => {
            if !code.is_empty() {
                prompt.code_block(&code, "");
            } else {
                prompt.info("No code generated");
            }
        }
        Err(e) => {
            prompt.error(&format!("Execution failed: {}", e));
        }
    }
    
    Ok(())
}

async fn analyze_file(client: MCPClient, config: Config, file: &str) -> Result<(), Box<dyn std::error::Error>> {
    let prompt = Prompt::new(config.cli.prompt, config.cli.color_output);
    
    if !std::path::Path::new(file).exists() {
        prompt.error(&format!("File not found: {}", file));
        return Ok(());
    }
    
    prompt.info(&format!("Analyzing file: {}", file));
    
    match client.analyze_file(file).await {
        Ok(analysis) => {
            prompt.success("Analysis complete:");
            println!("{}", analysis);
        }
        Err(e) => {
            prompt.error(&format!("Analysis failed: {}", e));
        }
    }
    
    Ok(())
}